@@ -15,17 +15,22 @@ using System.IO;
 using System.Reflection;
 using System.Threading;
 using System.Collections;
+using System.Collections.Generic;
 using System.Web.Services.Protocols;
 using System.Xml;
 
 namespace Monodoc {
 class Driver {
+	  
+	public static string[] engines = {"WebKit", "Gecko", "MonoWebBrowser", "GtkHtml"};
+	  
 	static int Main (string [] args)
 	{
 		string topic = null;
-		bool useGecko = true;
 		bool remote_mode = false;
 		
+		string engine = engines[0];
+		
 		for (int i = 0; i < args.Length; i++){
 			switch (args [i]){
 			case "--html":
@@ -98,8 +103,14 @@ class Driver {
 				remote_mode = true;
 				break;
 				
-			case "--no-gecko":
-				useGecko = false;
+			case "--engine":
+				if (i + 1 == args.Length) {
+					Console.WriteLine ("Usage: --engine engine, where engine is the name of the browser engine to use (WebKit, GtkHtml, MonoWebBrowser, Gecko or another).");
+					return 1;
+				}
+
+				engine = args [i+1];
+				i++;
 				break;
 			default:
 				topic = args [i];
@@ -112,7 +123,7 @@ class Driver {
 		
 		Settings.RunningGUI = true;
 		Application.Init ();
-		Browser browser = new Browser (useGecko);
+		Browser browser = new Browser (engine);
 		
 		if (topic != null)
 			browser.LoadUrl (topic);
@@ -165,7 +176,7 @@ public class Browser {
 	public Notebook tabs_nb;
 	public Tab CurrentTab;
 	bool HoldCtrl;
-	public bool UseGecko;
+	public string engine;
 
 	[Glade.Widget] public MenuItem bookmarksMenu;
 	[Glade.Widget] MenuItem view1;
@@ -231,9 +242,11 @@ public class Browser {
 
 	public ArrayList bookList;
 
-	public Browser (bool UseGecko)
+	public Capabilities capabilities;
+
+	public Browser (string engine)
 	{
-		this.UseGecko = UseGecko;
+		this.engine = engine;		
 		ui = new Glade.XML (null, "browser.glade", "window1", null);
 		ui.Autoconnect (this);
 
@@ -280,7 +293,10 @@ public class Browser {
 		tabs_nb.SwitchPage += new SwitchPageHandler(ChangeTab);
 		help_container.Add(tabs_nb);
 
-		if (UseGecko) {
+		AddTab();
+			
+			
+		if ((capabilities & Capabilities.Fonts) != 0) {
 			// Add Menu entries for changing the font
 			Menu aux = (Menu) view1.Submenu;
 			MenuItem sep = new SeparatorMenuItem ();
@@ -334,10 +350,11 @@ public class Browser {
 
 		index_browser = IndexBrowser.MakeIndexBrowser (this);
 		
-		AddTab();
 		MainWindow.ShowAll();
 	}
 
+
+
 	// Initianlizes the search index
 	void CreateSearchPanel ()
 	{
@@ -821,7 +838,7 @@ ExtLoop:
 	void on_print_activate (object sender, EventArgs e) 
 	{
 		 // desactivate css temporary
-		 if (UseGecko)
+		 if ((capabilities & Capabilities.Css) != 0)
 		 	HelpSource.use_css = false;
 		 
 		string url = CurrentUrl;
@@ -842,7 +859,7 @@ ExtLoop:
 		if (html != null)
 			CurrentTab.html.Print (html);
 
-		if (UseGecko)
+		if ((capabilities & Capabilities.Css) != 0)
 			HelpSource.use_css = true;
 	}
 
@@ -2264,19 +2281,55 @@ public class Tab : Notebook {
 			text_editor.GrabFocus ();	
 	}
 
-	static IHtmlRender GetRenderer (string file, string type, Browser browser)
-	{
-		try {
-			
-			string exeAssembly = Assembly.GetExecutingAssembly ().Location;
-			string myPath = System.IO.Path.GetDirectoryName (exeAssembly);
-			Assembly dll = Assembly.LoadFrom (System.IO.Path.Combine (myPath, file));
-			Type t = dll.GetType (type, true);
-		
-			return (IHtmlRender) Activator.CreateInstance (t, new object [1] { browser.help_tree });
-		} catch {
+
+	private static IHtmlRender LoadRenderer (string dll, Browser browser) {
+		if (!System.IO.File.Exists (dll))
 			return null;
+		
+		try {
+			Assembly ass = Assembly.LoadFile (dll);		
+			System.Type type = ass.GetType ("Monodoc." + ass.GetName ().Name, false, false);
+			if (type == null)
+				return null;
+			return (IHtmlRender) Activator.CreateInstance (type, new object[1] { browser.help_tree });
+		} catch (Exception ex) {
+			Console.Error.WriteLine (ex);
 		}
+		return null;
+	}
+	
+	
+	public static IHtmlRender GetRenderer (string engine, Browser browser)
+	{
+		IHtmlRender renderer = LoadRenderer (System.IO.Path.Combine (AppDomain.CurrentDomain.BaseDirectory, engine + "HtmlRender.dll"), browser);
+		if (renderer != null) {
+			try {
+				if (renderer.Initialize ()) {
+					Console.WriteLine ("using " + renderer.Name);
+					return renderer;
+				}
+			} catch (Exception ex) {
+				Console.Error.WriteLine (ex);
+			}
+		}
+		
+		foreach (string backend in Driver.engines) {
+			if (backend != engine) {
+				renderer = LoadRenderer (System.IO.Path.Combine (AppDomain.CurrentDomain.BaseDirectory, backend + "HtmlRender.dll"), browser);
+				if (renderer != null) {
+					try {
+						if (renderer.Initialize ()) {
+							Console.WriteLine ("using " + renderer.Name);
+							return renderer;
+						}
+					} catch (Exception ex) {
+						Console.Error.WriteLine (ex);
+					}
+				}			
+			}
+		}
+		
+		return null;		
 	}
 	
 
@@ -2300,24 +2353,19 @@ public class Tab : Notebook {
 		//
 		// Setup the HTML rendering and preview area
 		//
-		if (browser.UseGecko) {
-			html = GetRenderer ("GeckoHtmlRender.dll", "Monodoc.GeckoHtmlRender", browser);
-			html_preview = GetRenderer ("GeckoHtmlRender.dll", "Monodoc.GeckoHtmlRender", browser);
-			HelpSource.use_css = true;
-		}
-		
-		if (html == null || html_preview == null) {
-			html = GetRenderer ("GtkHtmlHtmlRender.dll", "Monodoc.GtkHtmlHtmlRender", browser);
-			html_preview = GetRenderer ("GtkHtmlHtmlRender.dll", "Monodoc.GtkHtmlHtmlRender", browser);
-			browser.UseGecko = false;
-			HelpSource.use_css = false;
-		}
 
+		html = GetRenderer (browser.engine, browser);
+		html_preview = GetRenderer (browser.engine, browser);
 		if (html == null || html_preview == null)
 			throw new Exception ("Couldn't find html renderer!");
-				
+
+		browser.capabilities = html.Capabilities;
+
+		if ((html.Capabilities & Capabilities.Css) != 0)
+			HelpSource.use_css = true;
+
 		//Prepare Font for css (TODO: use GConf?)
-		if (browser.UseGecko && SettingsHandler.Settings.preferred_font_size == 0) { 
+		if ((html.Capabilities & Capabilities.Fonts) != 0 && SettingsHandler.Settings.preferred_font_size == 0) { 
 			Pango.FontDescription font_desc = Pango.FontDescription.FromString ("Sans 12");
 			SettingsHandler.Settings.preferred_font_family = font_desc.Family;
 			SettingsHandler.Settings.preferred_font_size = 100; //size: 100%
