@@ -31,9 +31,12 @@
 // THE SOFTWARE.
 
 using System;
+using System.Collections.Generic;
 
 using Mono.Cecil;
 
+using Gendarme.Framework.Helpers;
+
 namespace Gendarme.Framework.Rocks {
 
 	// add Type[Definition|Reference] extensions methods here
@@ -53,28 +56,74 @@ namespace Gendarme.Framework.Rocks {
 	public static class TypeRocks {
 
 		/// <summary>
-		/// Return the type finalizer (aka destructor).
+		/// Return an IEnumerable that allows a single loop (like a foreach) to
+		/// traverse all MethodDefinition in the type. That includes the Constructors
+		/// and Methods collections.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
-		/// <returns>The MethodDefinition of the finalizer or null if the type has no finalizer.</returns>
-		[Obsolete ("use the more general GetMethod (MethodSignatures.Finalize)")]
-		public static MethodDefinition GetFinalizer (this TypeDefinition self)
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
+		/// <returns>An IEnumerable to traverse all constructors and methods</returns>
+		public static IEnumerable<MethodDefinition> AllMethods (this TypeReference self)
 		{
-			return self.GetMethod (MethodSignatures.Finalize);
+			TypeDefinition type = self.Resolve ();
+			foreach (MethodDefinition ctor in type.Constructors)
+				yield return ctor;
+			foreach (MethodDefinition method in type.Methods)
+				yield return method;
+		}
+
+		/// <summary>
+		/// Check if a type reference collection contains a type of a specific name.
+		/// </summary>
+		/// <param name="self">The TypeReferenceCollection on which the extension method can be called.</param>
+		/// <param name="typeName">Full name of the type.</param>
+		/// <returns>True if the collection contains an type of the same name,
+		/// False otherwise.</returns>
+		public static bool ContainsType (this TypeReferenceCollection self, string typeName)
+		{
+			if (typeName == null)
+				throw new ArgumentNullException ("typeName");
+
+			foreach (TypeReference type in self) {
+				if (type.FullName == typeName)
+					return true;
+			}
+			return false;
+		}
+
+		/// <summary>
+		/// Check if a type reference collection contains any of the specified type names.
+		/// </summary>
+		/// <param name="self">The TypeReferenceCollection on which the extension method can be called.</param>
+		/// <param name="typeNames">A string array of full type names.</param>
+		/// <returns>True if the collection contains any types matching one specified,
+		/// False otherwise.</returns>
+		public static bool ContainsAnyType (this TypeReferenceCollection self, string [] typeNames)
+		{
+			if (typeNames == null)
+				throw new ArgumentNullException ("typeNames");
+
+			foreach (TypeReference type in self) {
+				string fullname = type.FullName;
+				foreach (string type_full_name in typeNames) {
+					if (fullname == type_full_name)
+						return true;
+				}
+			}
+			return false;
 		}
 
 		/// <summary>
 		/// Returns the first MethodDefinition that satisfies a given MethodSignature.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <param name="signature">The MethodSignature to match.</param>
 		/// <returns>The first MethodDefinition for wich signature.Matches returns true.</returns>
 		/// <remarks>
 		/// Do not allocate a MethodSignature for only one call. Use one of the other GetMethod overloads instead.
 		/// </remarks>
-		public static MethodDefinition GetMethod (this TypeDefinition self, MethodSignature signature)
+		public static MethodDefinition GetMethod (this TypeReference self, MethodSignature signature)
 		{
-			foreach (MethodDefinition method in self.Methods) {
+			foreach (MethodDefinition method in self.AllMethods ()) {
 				if (signature.Matches (method))
 					return method;
 			}
@@ -84,16 +133,16 @@ namespace Gendarme.Framework.Rocks {
 		/// <summary>
 		/// Searches for a method.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <param name="attributes">An attribute mask matched against the attributes of the method.</param>
 		/// <param name="name">The name of the method to match. Ignored if null.</param>
 		/// <param name="returnType">The full name (Namespace.Type) of the return type. Ignored if null.</param>
 		/// <param name="parameters">An array of full names (Namespace.Type) of parameter types. Ignored if null. Null entries act as wildcards.</param>
 		/// <param name="customCondition">A custom condition that is called for each MethodDefinition that satisfies all other conditions. Ignored if null.</param>
 		/// <returns>The first MethodDefinition that satisfies all conditions.</returns>
-		public static MethodDefinition GetMethod (this TypeDefinition self, MethodAttributes attributes, string name, string returnType, string [] parameters, Func<MethodDefinition, bool> customCondition)
+		public static MethodDefinition GetMethod (this TypeReference self, MethodAttributes attributes, string name, string returnType, string [] parameters, Func<MethodDefinition, bool> customCondition)
 		{
-			foreach (MethodDefinition method in self.Methods) {
+			foreach (MethodDefinition method in self.AllMethods ()) {
 				if (name != null && method.Name != name)
 					continue;
 				if ((method.Attributes & attributes) != attributes)
@@ -107,13 +156,13 @@ namespace Gendarme.Framework.Rocks {
 					for (int i = 0; i < parameters.Length; i++) {
 						if (parameters [i] == null)
 							continue;//ignore parameter
-						if (parameters [i] != method.Parameters [i].ParameterType.FullName) {
+						if (parameters [i] != method.Parameters [i].ParameterType.GetOriginalType ().FullName) {
 							parameterError = true;
 							break;
 						}
 					}
 					if (parameterError)
-						break;
+						continue; // there could be an overload with the "right" parameters
 				}
 				if (customCondition != null && !customCondition (method))
 					continue;
@@ -125,13 +174,13 @@ namespace Gendarme.Framework.Rocks {
 		/// <summary>
 		/// Searches for a method by name, returnType, parameters and attributes.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <param name="attributes">An attribute mask matched against the attributes of the method.</param>
 		/// <param name="name">The name of the method to match. Ignored if null.</param>
 		/// <param name="returnType">The full name (Namespace.Type) of the return type. Ignored if null.</param>
 		/// <param name="parameters">An array of full names (Namespace.Type) of parameter types. Ignored if null. Null entries act as wildcard.</param>
 		/// <returns>The first MethodDefinition that satisfies all conditions.</returns>
-		public static MethodDefinition GetMethod (this TypeDefinition self, MethodAttributes attributes, string name, string returnType, string [] parameters)
+		public static MethodDefinition GetMethod (this TypeReference self, MethodAttributes attributes, string name, string returnType, string [] parameters)
 		{
 			return self.GetMethod (attributes, name, returnType, parameters, null);
 		}
@@ -139,11 +188,11 @@ namespace Gendarme.Framework.Rocks {
 		/// <summary>
 		/// Searches for a method by attributes and by name.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <param name="attributes">An attribute mask matched against the attributes of the method.</param>
 		/// <param name="name">The name of the method to match. Ignored if null.</param>
 		/// <returns>The first MethodDefinition that satisfies all conditions.</returns>
-		public static MethodDefinition GetMethod (this TypeDefinition self, MethodAttributes attributes, string name)
+		public static MethodDefinition GetMethod (this TypeReference self, MethodAttributes attributes, string name)
 		{
 			return self.GetMethod (attributes, name, null, null, null);
 		}
@@ -151,12 +200,12 @@ namespace Gendarme.Framework.Rocks {
 		/// <summary>
 		/// Searches for a method by name, returnType and parameters.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <param name="name">The name of the method to match. Ignored if null.</param>
 		/// <param name="returnType">The full name (Namespace.Type) of the return type. Ignored if null.</param>
 		/// <param name="parameters">An array of full names (Namespace.Type) of parameter types. Ignored if null. Null entries act as wildcards.</param>
 		/// <returns>The first MethodDefinition that satisfies all conditions.</returns>
-		public static MethodDefinition GetMethod (this TypeDefinition self, string name, string returnType, string [] parameters)
+		public static MethodDefinition GetMethod (this TypeReference self, string name, string returnType, string [] parameters)
 		{
 			return self.GetMethod (0, name, returnType, parameters, null);
 		}
@@ -164,10 +213,10 @@ namespace Gendarme.Framework.Rocks {
 		/// <summary>
 		/// Searches for a method with a specific name.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <param name="name">The name of the method to match.</param>
 		/// <returns>The first MethodDefinition with a specifiy name.</returns>
-		public static MethodDefinition GetMethod (this TypeDefinition self, string name)
+		public static MethodDefinition GetMethod (this TypeReference self, string name)
 		{
 			return self.GetMethod (0, name, null, null, null);
 		}
@@ -175,33 +224,21 @@ namespace Gendarme.Framework.Rocks {
 		/// <summary>
 		/// Searches for a method using a custom condition.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <param name="customCondition">A custom condition that is called for each MethodDefinition.</param>
 		/// <returns>The first MethodDefinition that satisfies the customCondition.</returns>
-		public static MethodDefinition GetMethod (this TypeDefinition self, Func<MethodDefinition, bool> customCondition)
+		public static MethodDefinition GetMethod (this TypeReference self, Func<MethodDefinition, bool> customCondition)
 		{
 			return self.GetMethod (0, null, null, null, customCondition);
 		}
 
-		/// <summary>
-		/// Check if the type contains an attribute of a specified type.
-		/// </summary>
-		/// <param name="self">The TypeReference on which the extension method can be called.</param>
-		/// <param name="attributeName">Full name of the attribute class</param>
-		/// <returns>True if the type contains an attribute of the same name,
-		/// False otherwise.</returns>
-		public static bool HasAttribute (this TypeReference self, string attributeName)
-		{
-			return self.CustomAttributes.ContainsType (attributeName);
-		}
-
 		/// <summary>
 		/// Checks if at least one Method satisfies a given MethodSignature.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <param name="signature">The MethodSignature to match.</param>
 		/// <returns>True if at least one method matches the signature. Otherwise false.</returns>
-		public static bool HasMethod (this TypeDefinition self, MethodSignature signature)
+		public static bool HasMethod (this TypeReference self, MethodSignature signature)
 		{
 			return (self.GetMethod (signature) != null);
 		}
@@ -214,24 +251,30 @@ namespace Gendarme.Framework.Rocks {
 		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
 		/// <param name="interfaceName">Full name of the interface</param>
 		/// <returns>True if the type implements the interface, False otherwise.</returns>
-		public static bool Implements (this TypeDefinition self, string interfaceName)
+		public static bool Implements (this TypeReference self, string interfaceName)
 		{
 			if (interfaceName == null)
 				throw new ArgumentNullException ("interfaceName");
 
+			bool generic = (interfaceName.IndexOf ('`') >= 0);
+
+			TypeDefinition type = self.Resolve ();
+			if (type == null)
+				return false;		// could not resolve
+
 			// special case, check if we implement ourselves
-			if (self.IsInterface && (self.FullName == interfaceName))
+			if (type.IsInterface && (type.FullName == interfaceName))
 				return true;
 
 			// does the type implements it itself
-			foreach (TypeReference iface in self.Interfaces) {
-				if (iface.FullName == interfaceName)
+			foreach (TypeReference iface in type.Interfaces) {
+				string fullname = (generic) ? iface.GetOriginalType ().FullName : iface.FullName;
+				if (fullname == interfaceName)
 					return true;
 			}
 			
 			// if not, then maybe it's parent does
-			// FIXME: right now we "ignore" case were a TypeReference is given
-			TypeDefinition parent = (self.BaseType as TypeDefinition);
+			TypeReference parent = type.BaseType;
 			if (parent != null)
 				return parent.Implements (interfaceName);
 
@@ -240,7 +283,7 @@ namespace Gendarme.Framework.Rocks {
 
 		/// <summary>
 		/// Check if the type inherits from the specified type. Note that it is possible that
-		/// we might now be able to know the complete inheritance chain since the assembly 
+		/// we might not be able to know the complete inheritance chain since the assembly 
 		/// where the information resides could be unavailable.
 		/// </summary>
 		/// <param name="self">The TypeReference on which the extension method can be called.</param>
@@ -251,14 +294,15 @@ namespace Gendarme.Framework.Rocks {
 			if (className == null)
 				throw new ArgumentNullException ("className");
 
-			TypeReference current = self;
+			TypeReference current = self.Resolve ();
 			while ((current != null) && (current.FullName != "System.Object")) {
 				if (current.FullName == className)
 					return true;
 
-				// FIXME: plugin AssemblyResolver when ready
-				TypeDefinition type = (current as TypeDefinition);
-				current = (type == null) ? null : type.BaseType;
+				TypeDefinition td = current.Resolve ();
+				if (td == null)
+					return false;		// could not resolve type
+				current = td.BaseType;
 			}
 			return false;
 		}
@@ -278,25 +322,47 @@ namespace Gendarme.Framework.Rocks {
 		/// we might now be able to know all inheritance since the assembly where 
 		/// the information resides could be unavailable.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <returns>True if the type inherits from <c>System.Attribute</c>, 
 		/// False otherwise.</returns>
-		public static bool IsAttribute (this TypeDefinition self)
+		public static bool IsAttribute (this TypeReference self)
 		{
 			return self.Inherits ("System.Attribute");
 		}
 
 		/// <summary>
-		/// Check if the type is a enumeration flags.
+		/// Check if the type is a delegate.
 		/// </summary>
-		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
-		/// <returns>True if the type as the [Flags] attribute, false otherwise.</returns>
-		public static bool IsFlags (this TypeDefinition self)
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
+		/// <returns>True if the type is a delegate, False otherwise.</returns>
+		public static bool IsDelegate (this TypeReference self)
 		{
-			if (!self.IsEnum)
+			TypeDefinition type = self.Resolve ();
+			// e.g. this occurs for <Module> or GenericParameter
+			if (null == type || type.BaseType == null)
 				return false;
 
-			return self.HasAttribute ("System.FlagsAttribute");
+			switch (type.BaseType.FullName) {
+			case "System.Delegate":
+			case "System.MulticastDelegate":
+				return true;
+			default:
+				return false;
+			}
+		}
+
+		/// <summary>
+		/// Check if the type is a enumeration flags.
+		/// </summary>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
+		/// <returns>True if the type as the [Flags] attribute, false otherwise.</returns>
+		public static bool IsFlags (this TypeReference self)
+		{
+			TypeDefinition type = self.Resolve ();
+			if (!type.IsEnum)
+				return false;
+
+			return type.HasAttribute ("System.FlagsAttribute");
 		}
 
 		/// <summary>
@@ -322,12 +388,13 @@ namespace Gendarme.Framework.Rocks {
 			if (self.Module.Assembly.Runtime >= TargetRuntime.NET_2_0) {
 				if (self.CustomAttributes.ContainsAnyType (CustomAttributeRocks.GeneratedCodeAttributes))
 					return true;
-			} else {
-				switch (self.Name [0]) {
-				case '<': // e.g. <PrivateImplementationDetails>
-				case '$': // e.g. $ArrayType$1 nested inside <PrivateImplementationDetails>
-					return true;
-				}
+			}
+
+			// sadly <Module> still shows up for 2.0, so the 1.x logic still applies
+			switch (self.Name [0]) {
+			case '<': // e.g. <Module>, <PrivateImplementationDetails>
+			case '$': // e.g. $ArrayType$1 nested inside <PrivateImplementationDetails>
+				return true;
 			}
 
 			// the type could be nested (inside a generated one) and not marked itself
@@ -353,20 +420,54 @@ namespace Gendarme.Framework.Rocks {
 			}
 		}
 
+		/// <summary>
+		/// Check if the type is static (2.0+)
+		/// </summary>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
+		/// <returns>True if the type is static, false otherwise.</returns>
+		public static bool IsStatic (this TypeReference self)
+		{
+			TypeDefinition type = self.Resolve ();
+			if (type == null)
+				return false;
+			return (type.IsSealed && type.IsAbstract);
+		}
+
 		/// <summary>
 		/// Check if the type is visible outside of the assembly.
 		/// </summary>
 		/// <param name="self">The TypeReference on which the extension method can be called.</param>
 		/// <returns>True if the type can be used from outside of the assembly, false otherwise.</returns>
-		public static bool IsVisible (this TypeDefinition self)
+		public static bool IsVisible (this TypeReference self)
 		{
-			while (self.IsNested) {
-				if (self.IsNestedPrivate || self.IsNestedAssembly)
+			TypeDefinition type = self.Resolve ();
+			if (type == null)
+				return true; // it's probably visible since we have a reference to it
+
+			while (type.IsNested) {
+				if (type.IsNestedPrivate || type.IsNestedAssembly)
 					return false;
 				// Nested classes are always inside the same assembly, so the cast is ok
-				self = (TypeDefinition) self.DeclaringType;
+				type = type.DeclaringType.Resolve ();
 			}
-			return self.IsPublic;
+			return type.IsPublic;
+		}
+
+		/// <summary>
+		/// Resolve a TypeReference into a TypeDefinition.
+		/// </summary>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
+		/// <returns>A TypeDefinition if resolved, null otherwise.</returns>
+		public static TypeDefinition Resolve (this TypeReference self)
+		{
+			// this can occurs, e.g. generic parameters that needs recursive resolves
+			if (self == null)
+				return null;
+
+			TypeDefinition type = (self as TypeDefinition);
+			if (type == null)
+				type = AssemblyResolver.Resolver.Resolve (self);
+			return type;
 		}
 	}
 }
