@@ -230,13 +230,18 @@ namespace GuiCompare {
 			List<CompNamed> rv = new List<CompNamed>();
 			foreach (CustomAttribute ca in provider.CustomAttributes) {
 				TypeDefinition resolved = CecilUtils.Resolver.Resolve (ca.Constructor.DeclaringType);
-				
-				if (resolved.IsNotPublic)
-					continue;
-				
-				if (IsTODOAttribute (resolved))
-					todos.Add (String.Format ("[{0} ({1})]", ca.Constructor.DeclaringType.Name, CecilUtils.GetTODOText (ca)));
-				else if (!ShouldSkipAttribute (ca.Constructor.DeclaringType.FullName))
+
+				if (resolved != null) {
+					if (resolved.IsNotPublic)
+						continue;
+
+					if (IsTODOAttribute (resolved)) {
+						todos.Add (String.Format ("[{0} ({1})]", ca.Constructor.DeclaringType.Name, CecilUtils.GetTODOText (ca)));					
+						continue;
+					}
+				}
+
+				if (!ShouldSkipAttribute (ca.Constructor.DeclaringType.FullName))
 					rv.Add (new CecilAttribute (ca));
 			}
 			return rv;
@@ -650,7 +655,13 @@ namespace GuiCompare {
 		                                             FieldAttributes.HasFieldMarshal);
 		public override string GetMemberAccess ()
 		{
-			return (field_def.Attributes & masterInfoFieldMask).ToString();
+			FieldAttributes fa = field_def.Attributes & masterInfoFieldMask;
+
+			// remove the Assem from FamORAssem
+			if ((fa & FieldAttributes.FamORAssem) == FieldAttributes.FamORAssem)
+				fa = (fa & ~(FieldAttributes.FamORAssem)) | (FieldAttributes.Family);
+
+			return fa.ToString();
 		}
 		
 		public override List<CompNamed> GetAttributes ()
@@ -706,7 +717,13 @@ namespace GuiCompare {
 		                                               MethodAttributes.SpecialName);
 		public override string GetMemberAccess ()
 		{
-			return (method_def.Attributes & masterInfoMethodMask).ToString();
+			MethodAttributes ma = method_def.Attributes & masterInfoMethodMask;
+
+			// remove the Assem from FamORAssem
+			if ((ma & MethodAttributes.FamORAssem) == MethodAttributes.FamORAssem)
+				ma = (ma & ~(MethodAttributes.FamORAssem)) | (MethodAttributes.Family);
+
+			return ma.ToString();
 		}
 		
 		public override List<CompNamed> GetAttributes ()
@@ -798,10 +815,11 @@ namespace GuiCompare {
 	public class CecilProperty : CompProperty
 	{
 		public CecilProperty (PropertyDefinition pd)
-			: base (pd.Name)
+			: base (FormatName (pd, false))
 		{
 			this.pd = pd;
 			this.attributes = CecilUtils.GetCustomAttributes (pd, todos);
+			this.DisplayName = FormatName (pd, true);
 		}
 
 		public override string GetMemberType()
@@ -830,6 +848,44 @@ namespace GuiCompare {
 			
 			return rv;
 		}
+
+		static string FormatName (PropertyDefinition pd, bool beautify)
+		{
+			StringBuilder sb = new StringBuilder ();
+
+#if INCLUDE_TYPE_IN_PROPERTY_DISPLAYNAME
+			sb.Append (beautify
+				           ? CecilUtils.PrettyType (pd.PropertyType.FullName)
+				           : CecilUtils.FormatTypeLikeCorCompare (pd.PropertyType.FullName));
+			sb.Append (" ");
+#else
+			if (!beautify) {
+				sb.Append (CecilUtils.FormatTypeLikeCorCompare (pd.PropertyType.FullName));
+				sb.Append (" ");
+			}
+#endif
+			sb.Append (pd.Name);
+
+			if (pd.Parameters.Count > 0) {
+				sb.Append ('[');
+				bool first_p = true;
+				foreach (ParameterDefinition p in pd.Parameters) {
+					if (!first_p)
+						sb.Append (", ");
+					first_p = false;
+					sb.Append (beautify
+						   ? CecilUtils.PrettyType (p.ParameterType.FullName)
+						   : CecilUtils.FormatTypeLikeCorCompare (p.ParameterType.FullName));
+					if (beautify) {
+						sb.Append (" ");
+						sb.Append (p.Name);
+					}
+				}
+				sb.Append (']');
+			}
+
+			return sb.ToString ();
+		}
 		
 		PropertyDefinition pd;
 		List<CompNamed> attributes;
